#include <iostream>
#include <cmath>
#include <algorithm>
#include <string>
#include <vector>
#include <set>
#include <numeric>
#include <map>

using namespace std;

using ll = long long;

void findCycle(int v, int p, vector<bool> &used, vector<vector<pair<int, ll>>> &adj, vector<int> &cycle)
{
    if (used[v])
    {
        cycle.push_back(v);
        return;
    }
    used[v] = true;
    for (auto [u, w] : adj[v])
    {
        if (p == u) continue;
        findCycle(u, v, used, adj, cycle);
        if (!cycle.empty())
        {
            if (cycle.size() == 1 || cycle.front() != cycle.back()) cycle.push_back(v);
            return;
        }
    }
}

ll dfs(int v, int p, vector<vector<pair<int, ll>>> &adj, vector<bool> &used)
{
    ll maxim = 0;
    for (auto [u, w] : adj[v]) if (u != p && !used[u]) maxim = max(maxim, dfs(u, v, adj, used) + w);
    return maxim;
}

struct segtree
{
    
    struct node
    {
        ll left, right;
        ll a;
    };
    
    vector<node> tree;
    vector<pair<ll, ll>> mod;
    
    node merge(node a, node b)
    {
        node res;
        if (max(a.left, a.right) + a.a > max(b.left, b.right) + b.a) res = a;
        else res = b;
        return res;
    }
    
    void push(int v)
    {
        if (mod[v].first != 0)
        {
            mod[v << 1].first += mod[v].first;
            mod[v << 1 | 1].first += mod[v].first;
            tree[v << 1].left += mod[v].first;
            tree[v << 1 | 1].left += mod[v].first;
            mod[v].first = 0;
        }
        if (mod[v].second != 0)
        {
            mod[v << 1].second += mod[v].second;
            mod[v << 1 | 1].second += mod[v].second;
            tree[v << 1].right += mod[v].second;
            tree[v << 1 | 1].right += mod[v].second;
            mod[v].second = 0;
        }
    }
    
    void build(int v, int l, int r, vector<node> &a)
    {
        if (l > r) return;
        if (l == r)
        {
            tree[v] = a[l];
            return;
        }
        int m = (l + r) >> 1;
        build(v << 1, l, m, a);
        build(v << 1 | 1, m + 1, r, a);
        tree[v] = merge(tree[v << 1], tree[v << 1 | 1]);
    }
    
    segtree(vector<ll> a, vector<ll> eL, vector<ll> eR)
    {
        vector<node> aa(a.size());
        for (int i = 0; i < a.size(); ++i)
        {
            aa[i].a = a[i];
            aa[i].left = eL[i];
            aa[i].right = eR[i];
        }
        tree.resize(4 * (a.size()) + 5);
        mod.resize(4 * (a.size()) + 5);
    }
    
    void update(pair<ll, ll> val)
    {
        tree[0].left += val.first;
        tree[0].right += val.second;
        mod[0].first += val.first;
        mod[0].second += val.second;
    }
    
};

int main()
{
    cin.tie(0)->sync_with_stdio(0);
    int n, g;
    cin >> n >> g;
    vector<vector<pair<int, ll>>> adj(n + 1);
    for (int i = 0; i < n; ++i)
    {
        int a, b;
        ll c;
        cin >> a >> b >> c;
        adj[a].emplace_back(b, c);
        adj[b].emplace_back(a, c);
    }
    vector<bool> used(n + 1);
    vector<int> cycle;
    findCycle(1, 0, used, adj, cycle);
    cycle.pop_back();
    used = vector<bool>(n + 1, 0);
    for (int i = 0; i < cycle.size(); ++i) used[cycle[i]] = 1;
    vector<ll> arr(cycle.size());
    for (int i = 0; i < cycle.size(); ++i) arr[i] = dfs(cycle[i], 0, adj, used);
    vector<ll> edges(cycle.size());
    for (int i = 0; i < cycle.size(); ++i)
    {
        int next = cycle[(i + 1) % cycle.size()];
        ll wei = -1;
        for (auto [u, w] : adj[cycle[i]])
        {
            if (u == next)
            {
                wei = w;
                break;
            }
        }
        edges[i] = wei;
    }
    multiset<pair<ll, ll>> left, right;
    ll dl = 0, dr = 0;
    ll sum = accumulate(edges.begin(), edges.end(), 0ll);
    ll dist = 0;
    int ind = -1;
    vector<ll> distt(cycle.size());
    for (int i = 0; i < cycle.size(); ++i)
    {
        if (dist < sum - dist) right.insert({dist + arr[i], i});
        else
        {
            left.insert({sum - dist + arr[i], i});
            if (ind == -1) ind = i;
        }
        distt[i] = dist;
        dist += edges[i];
    }
    ll maxRes = 0;
    for (int i = 0; i < cycle.size(); ++i)
    {
        maxRes = max({maxRes, left.rbegin()->first + dl, right.rbegin()->first + dr});
        //distt[i] = sum;
        right.erase({-dr, i});
        left.insert({-dl, i});
        dr -= edges[i];
        dl += edges[i];
        if (ind == -1) ind = i;
        while (!left.empty())
        {
            auto [w, i] = *left.rbegin();
            w -= arr[i];
            if (w + dl > sum - w + dr)
            {
                left.erase({w + arr[i], i});
                right.insert({sum - w - dl + arr[i] - dr, i});
            }
            else break;
        }
        if (left.empty()) ind = -1;
    }
    cout << maxRes;
    return 0;
}
