#include <iostream>
#include <cmath>
#include <algorithm>
#include <string>
#include <vector>
#include <set>
#include <numeric>
#include <map>
#include <queue>
#include <cassert>

using namespace std;

using ll = long long;

void findCycle(int v, int p, vector<bool> &used, vector<vector<pair<int, ll>>> &adj, vector<int> &cycle)
{
    if (used[v])
    {
        cycle.push_back(v);
        return;
    }
    used[v] = true;
    for (auto [u, w] : adj[v])
    {
        if (p == u) continue;
        findCycle(u, v, used, adj, cycle);
        if (!cycle.empty())
        {
            if (cycle.size() == 1 || cycle.front() != cycle.back()) cycle.push_back(v);
            return;
        }
    }
}

pair<int, ll> dfs(int v, int p, vector<vector<pair<int, ll>>> &adj, vector<bool> &used)
{
    ll maxim = 0;
    int who = v;
    for (auto [u, w] : adj[v])
    {
        if (u != p && !used[u])
        {
            auto [who1, wei] = dfs(u, v, adj, used);
            wei += w;
            if (wei > maxim)
            {
                maxim = wei;
                who = who1;
            }
        }
    }
    return {who, maxim};
}

ll dijkstra(int v, vector<vector<pair<int, ll>>> &adj)
{
    priority_queue<pair<ll, int>, vector<pair<ll, int>>, greater<>> pq;
    pq.push({0, v});
    vector<bool> used(adj.size());
    vector<ll> d(adj.size(), 1e18);
    d[v] = 0;
    while (!pq.empty())
    {
        int v = pq.top().second;
        pq.pop();
        if (used[v]) continue;
        used[v] = true;
        for (auto [u, w] : adj[v])
        {
            if (d[u] > d[v] + w)
            {
                d[u] = d[v] + w;
                pq.push({d[u], u});
            }
        }
    }
    ll maxim = 0;
    for (int i = 1; i < adj.size(); ++i) maxim = max(maxim, d[i]);
    return maxim;
}

int main()
{
    cin.tie(0)->sync_with_stdio(0);
    int n, g;
    cin >> n >> g;
    vector<vector<pair<int, ll>>> adj(n + 1);
    for (int i = 0; i < n; ++i)
    {
        int a, b;
        ll c;
        cin >> a >> b >> c;
        adj[a].emplace_back(b, c);
        adj[b].emplace_back(a, c);
    }
    vector<bool> used(n + 1);
    vector<int> cycle;
    findCycle(1, 0, used, adj, cycle);
    cycle.pop_back();
    used = vector<bool>(n + 1, 0);
    for (int i = 0; i < cycle.size(); ++i) used[cycle[i]] = 1;
    vector<ll> arr(cycle.size());
    ll maxRes = 0;
    for (int i = 0; i < cycle.size(); ++i)
    {
        auto [v, w] = dfs(cycle[i], 0, adj, used);
        arr[i] = w;
        used[cycle[i]] = false;
        maxRes = max(maxRes, dfs(v, 0, adj, used).second);
        used[cycle[i]] = true;
    }
    vector<ll> edges(cycle.size());
    for (int i = 0; i < cycle.size(); ++i)
    {
        int next = cycle[(i + 1) % cycle.size()];
        for (auto [u, w] : adj[cycle[i]])
        {
            if (u == next)
            {
                edges[i] = w;
                break;
            }
        }
    }
    multiset<pair<ll, ll>> left, right;
    ll dl = 0, dr = 0;
    ll dist = 0;
    int ind = -1;
    vector<ll> distt(cycle.size());
    ll sum = accumulate(edges.begin(), edges.end(), 0ll);
    for (int i = 0; i < cycle.size(); ++i)
    {
        if (dist < sum - dist) right.insert({dist + arr[i], i});
        else
        {
            left.insert({sum - dist + arr[i], i});
            if (ind == -1) ind = i;
        }
        distt[i] = dist;
        dist += edges[i];
    }
    for (int i = 0; i < cycle.size(); ++i)
    {
        right.erase({-dr, i});
        maxRes = max({maxRes, left.rbegin()->first + dl + arr[i], right.rbegin()->first + dr + arr[i]});
        //distt[i] = sum;
        left.insert({-dl, i});
        dr -= edges[i];
        dl += edges[i];
        while (!left.empty())
        {
            auto [w, i] = *left.rbegin();
            w -= arr[i];
            if (w + dl > sum - w + dr)
            {
                left.erase({w + arr[i], i});
                right.insert({sum - w - dl + arr[i] - dr, i});
            }
            else break;
        }
        /*ll d = edges[i];
        maxRes = max(maxRes, arr[i]);
        for (int j = (i + 1) % cycle.size(); j != i; j = (j + 1) % cycle.size())
        {
            maxRes = max(maxRes, min(sum - d, d) + arr[j] + arr[i]);
            d += edges[j];
        }*/
    }
    cout << maxRes;
    //happens that maxRes < maxRes2
    /*ll maxRes2 = 0;
    for (int i = 1; i <= n; ++i) maxRes2 = max(maxRes2, dijkstra(i, adj));
    assert(maxRes <= maxRes2);
    cout << maxRes2;*/
    return 0;
}
/*
 6 0
 1 2 1
 1 3 2
 2 3 10
 2 5 8
 3 4 6
 6 1 7
 */
