#include <iostream>
#include <cmath>
#include <algorithm>
#include <string>
#include <vector>
#include <set>
#include <numeric>
#include <map>
#include <queue>
#include <cassert>

using namespace std;

using ll = long long;

void findCycle(int v, int p, vector<bool> &used, vector<vector<pair<int, ll>>> &adj, vector<int> &cycle)
{
    if (used[v])
    {
        cycle.push_back(v);
        return;
    }
    used[v] = true;
    for (auto [u, w] : adj[v])
    {
        if (p == u) continue;
        findCycle(u, v, used, adj, cycle);
        if (!cycle.empty())
        {
            if (cycle.size() == 1 || cycle.front() != cycle.back()) cycle.push_back(v);
            return;
        }
    }
}

pair<int, ll> dfs(int v, int p, vector<vector<pair<int, ll>>> &adj, vector<bool> &used)
{
    ll maxim = 0;
    int who = v;
    for (auto [u, w] : adj[v])
    {
        if (u != p && !used[u])
        {
            auto [who1, wei] = dfs(u, v, adj, used);
            wei += w;
            if (wei > maxim)
            {
                maxim = wei;
                who = who1;
            }
        }
    }
    return {who, maxim};
}

int main()
{
    cin.tie(0)->sync_with_stdio(0);
    int n, g;
    cin >> n >> g;
    vector<vector<pair<int, ll>>> adj(n + 1);
    for (int i = 0; i < n; ++i)
    {
        int a, b;
        ll c;
        cin >> a >> b >> c;
        adj[a].emplace_back(b, c);
        adj[b].emplace_back(a, c);
    }
    vector<bool> used(n + 1);
    vector<int> cycle;
    findCycle(1, 0, used, adj, cycle);
    cycle.pop_back();
    used = vector<bool>(n + 1, 0);
    for (int i = 0; i < cycle.size(); ++i) used[cycle[i]] = 1;
    vector<ll> arr(cycle.size());
    ll maxRes = 0;
    for (int i = 0; i < cycle.size(); ++i)
    {
        auto [v, w] = dfs(cycle[i], 0, adj, used);
        arr[i] = w;
        used[cycle[i]] = false;
        maxRes = max(maxRes, dfs(v, 0, adj, used).second);
        used[cycle[i]] = true;
    }
    vector<ll> edges(cycle.size());
    for (int i = 0; i < cycle.size(); ++i)
    {
        int next = cycle[(i + 1) % cycle.size()];
        for (auto [u, w] : adj[cycle[i]])
        {
            if (u == next)
            {
                edges[i] = w;
                break;
            }
        }
    }
    int ind = cycle.size();
    multiset<ll> left, right;
    vector<ll> dist(cycle.size());
    ll d = 0;
    ll sum = accumulate(edges.begin(), edges.end(), 0ll);
    for (int i = 0; i < cycle.size(); ++i)
    {
        dist[i] = d;
        if (dist[i] < sum - dist[i]) right.insert(dist[i] + arr[i]);
        else
        {
            left.insert(sum - dist[i] + arr[i]);
            if (ind == cycle.size()) ind = i;
        }
        d += edges[i];
    }
    d = 0;
    for (int i = 0; i < cycle.size(); ++i)
    {
        right.erase(right.find(dist[i] + arr[i]));
        if (!right.empty()) maxRes = max(maxRes, *right.rbegin() + arr[i] + d);
        if (!left.empty()) maxRes = max(maxRes, *left.rbegin() + arr[i] - d);
        d -= edges[i];
        while (ind < cycle.size())
        {
            if (dist[ind] + d < sum - dist[ind] - d)
            {
                left.erase(sum - dist[ind] + arr[ind]);
                right.insert(dist[ind] + arr[ind]);
                ++ind;
            }
            else break;
        }
    }
    cout << maxRes;
    return 0;
}
