#include <iostream>
#include <array>
#include <vector>

using namespace std;

struct segtree
{
    
    struct xorbasis
    {
        array<int, 20> basis;
        int size = 0;
        
        xorbasis()
        {
            basis.fill(-1);
        }
        
        int find(int k)
        {
            if (k > (1 << size)) return -1;
            --k;
            int res = 0, all = (1 << size);
            for (int i = 19; i >= 0; --i)
            {
                if (basis[i] == -1) continue;
                all >>= 1;
                if (k >= all)
                {
                    res ^= basis[i];
                    k -= all;
                }
            }
            return res;
        }
        
        void insert(int x)
        {
            for (int i = 0; i < 20; ++i)
            {
                if (basis[i] == -1) continue;
                x = min(x, x ^ basis[i]);
            }
            if (x > 0)
            {
                int bit = -1;
                for (int i = 19; i >= 0; --i)
                {
                    if (basis[i] != -1) basis[i] = min(basis[i], basis[i] ^ x);
                    if (x & (1 << i))
                    {
                        bit = i;
                        break;
                    }
                }
                basis[bit] = x;
                ++size;
            }
        }
        
    };
    
    vector<xorbasis> tree;
    int n;
    
    void merge(xorbasis a, xorbasis b, xorbasis &c)
    {
        c = b;
        for (int i = 0; i < 20; ++i) if (a.basis[i] != -1) c.insert(a.basis[i]);
    }
    
    void build(int v, int l, int r, vector<int> &a)
    {
        if (l > r) return;
        if (l == r)
        {
            tree[v].insert(a[l]);
            return;
        }
        int m = (l + r) >> 1;
        build(v << 1, l, m, a);
        build(v << 1 | 1, m + 1, r, a);
        merge(tree[v << 1], tree[v << 1 | 1], tree[v]);
    }
    
    segtree(vector<int> &a)
    {
        n = (int)a.size();
        tree.resize(a.size() * 4 + 1);
        build(1, 0, n - 1, a);
    }
    
    void update(int p, int val, int l, int r, int v)
    {
        if (l > r) return;
        if (l == r)
        {
            tree[v] = xorbasis();
            tree[v].insert(val);
            return;
        }
        int m = (l + r) >> 1;
        if (p <= m) update(p, val, l, m, v << 1);
        else update(p, val, m + 1, r, v << 1 | 1);
        merge(tree[v << 1], tree[v << 1 | 1], tree[v]);
    }
    
    xorbasis get(int a, int b, int l, int r, int v)
    {
        if (a > r || b < l) return xorbasis();
        if (a == l && b == r) return tree[v];
        int m = (l + r) >> 1;
        xorbasis ret;
        merge(get(a, min(b, m), l, m, v << 1), get(max(a, m + 1), b, m + 1, r, v << 1 | 1), ret);
        return ret;
    }
    
    int get(int a, int b, int k)
    {
        xorbasis ret = get(a, b, 0, n - 1, 1);
        return ret.find(k);
    }
    
};

void euler(int v, int p, int &timer, vector<vector<int>> &adj, vector<int> &tin, vector<int> &tout)
{
    ++timer;
    tin[v] = timer;
    for (int u : adj[v]) if (u != p) euler(u, v, timer, adj, tin, tout);
    tout[v] = timer;
}

int main()
{
    cin.tie(0)->sync_with_stdio(0);
    int n, g;
    cin >> n >> g;
    vector<vector<int>> adj(n + 1);
    for (int i = 0; i + 1 < n; ++i)
    {
        int x, y;
        cin >> x >> y;
        adj[x].push_back(y);
        adj[y].push_back(x);
    }
    vector<int> tin(n + 1), tout(n + 1);
    int timer = -1;
    euler(1, 0, timer, adj, tin, tout);
    vector<int> a(timer + 1);
    for (int i = 1; i <= n; ++i) cin >> a[tin[i]];
    segtree st(a);
    int q;
    cin >> q;
    while (q--)
    {
        int t;
        cin >> t;
        if (t == 1)
        {
            int x, y;
            cin >> x >> y;
            st.update(tin[x], y, 0, timer, 1);
        }
        else
        {
            int v, k;
            cin >> v >> k;
            cout << st.get(tin[v], tout[v], k) << '\n';
        }
    }
    return 0;
}
/*
5 0
1 2
1 5
2 3
2 4
4 2 3 1 2
2
1 3 4
2 3 2
*/
