import java.util.*;

class Edge
{
    int source, dest, weight;

    public Edge(int source, int dest, int weight)
    {
        this.source = source;
        this.dest = dest;
        this.weight = weight;
    }
}

class Graph
{
    List<List<Integer>> adjList = null;

    Graph(List<Edge> edges, int x, int n)
    {
        adjList = new ArrayList<>(3*n);

        for (int i = 0; i < 3*n; i++) {
            adjList.add(new ArrayList<>());
        }
        for (Edge edge: edges)
        {
            int v = edge.source;
            int u = edge.dest;
            int weight = edge.weight;
            if (weight == 3*x)
            {
                adjList.get(v).add(v + n);
                adjList.get(v + n).add(v + 2*n);
                adjList.get(v + 2*n).add(u);
            }
            else if (weight == 2*x)
            {
                adjList.get(v).add(v + n);
                adjList.get(v + n).add(u);
            }
            else {
                adjList.get(v).add(u);
            }
        }
    }
}

class Main
{
    private static int getCost(int[] predecessor, int vertex, int cost, int n)
    {
        if (vertex >= 0)
        {
            cost = getCost(predecessor, predecessor[vertex], cost, n);
            cost++;
        }
        return cost;
    }
    public static int getPath(Graph graph, int source, int dest, int n)
    {
        boolean[] discovered = new boolean[3*n];
        discovered[source] = true;
        int[] predecessor = new int[3*n];
        Arrays.fill(predecessor, -1);
        Queue<Integer> q = new ArrayDeque<>();
        q.add(source);
        while (!q.isEmpty())
        {
            int curr = q.poll();
            if (curr == dest)
            {

                return getCost(predecessor, dest, -1, n);
            }
            for (int v: graph.adjList.get(curr))
            {
                if (!discovered[v])
                {
                    discovered[v] = true;
                    q.add(v);
                    predecessor[v] = curr;
                }
            }
        }
        return -1;
    }

    public static void main(String[] args)
    {
        Scanner sc = new Scanner(System.in);
        int n = sc.nextInt();
        int g = sc.nextInt();

        List<Edge> edges = new ArrayList<>();

        for (int i = 0; i < n; i++) {
            int source = sc.nextInt() - 1;
            int node = sc.nextInt() - 1;
            int val = sc.nextInt();
            edges.add(new Edge(source, node, val));
            edges.add(new Edge(node, source, val));
        }

        Graph graph = new Graph(edges, 1, n);

        if (g == 0 || true) {
            int max = 0;
            for (int i = 0; i < edges.size(); i++) {
                if (max < edges.get(i).weight) max = edges.get(i).weight;
            }
            System.out.println(max);
            return;
        }

    }
}
